@@ -1,4 +1,24 @@
 
+## Authors 
+## Martin Schlather, schlather@math.uni-mannheim.de
+##
+##
+## Copyright (C) 2015 Martin Schlather
+##
+## This program is free software; you can redistribute it and/or
+## modify it under the terms of the GNU General Public License
+## as published by the Free Software Foundation; either version 3
+## of the License, or (at your option) any later version.
+##
+## This program is distributed in the hope that it will be useful,
+## but WITHOUT ANY WARRANTY; without even the implied warranty of
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
+##
+## You should have received a copy of the GNU General Public License
+## along with this program; if not, write to the Free Software
+## Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.  
+
 
 print.RFratiotest <- function(x, ...) {  
   if (!is.null(x$simu.ratios)) {
@@ -70,7 +90,7 @@ approx_test <- function(modellist, alpha) {
   for (m in 1:n) {
     if (class(modellist[[m]]) == "RF_fit") {
       df[m] <- modellist[[m]]$number.of.parameters
-      loglik[m] <- modellist[[m]]$ml$ml.value
+      loglik[m] <- modellist[[m]]$ml$likelihood
     } else if (class(modellist[[m]]) == "RFfit") {
       df[m] <- modellist[[m]]@number.of.parameters
       loglik[m] <- modellist[[m]]["ml"]@likelihood
@@ -88,7 +108,7 @@ approx_test_single <- function(model, method, alpha, modelinfo) {
   if (class(model) == "RF_fit") {
     submodels <- model$submodels
     df <-  model$number.of.parameters
-    loglik <- model[[method]]$ml.value
+    loglik <- model[[method]]$likelihood
     report <- model$report
     p.proj <- model$p.proj
     v.proj <- model$v.proj
@@ -268,12 +288,11 @@ approx_test_single <- function(model, method, alpha, modelinfo) {
 
 RFratiotest <-
   function(nullmodel, alternative,
-           x, y=NULL, z=NULL, T=NULL,  grid, data,
+           x, y=NULL, z=NULL, T=NULL,  grid=NULL, data,
            alpha,
            n = 5 / alpha, ## number of simulations to do
            seed = 0,
            lower=NULL, upper=NULL, 
-           bc_lambda, ## if missing then no BoxCox-Trafo
            methods, # "reml", "rml1"),
            sub.methods,
            ## "internal" : name should not be changed; should always be last
@@ -293,7 +312,7 @@ RFratiotest <-
                                   ..., general.seed=NA)
   on.exit(RFoptions(LIST=RFoptOld[[1]]))
   RFopt <- RFoptOld[[2]]
-  printlevel <- RFopt$general$printlevel
+  printlevel <- RFopt$basic$printlevel
   if (RFopt$general$modus_operandi == "neurotic")
     stop("crossvalidation is not a precise method")
   
@@ -318,38 +337,50 @@ RFratiotest <-
                    "delta.df", "p", "txt"
                    ), drop=FALSE]
       class(ats) <- "RFratiotest"
-      return(ats)
+      if (RFopt$general$returncall)
+        attr(ats, "call") <- as.character(deparse(match.call()))
+      attr(ats, "coord_system") <- c(orig=RFopt$coords$coord_system,
+                                     model=RFopt$coords$new_coord_system)
+    return(ats)
     } else {
       ats <- approx_test(list(nullmodel, alternative), alpha)
       class(ats) <- "RFratiotest"
-      return(ats)
+      if (RFopt$general$returncall)
+        attr(ats, "call") <- as.character(deparse(match.call()))
+      attr(ats, "coord_system") <- c(orig=RFopt$coords$coord_system,
+                                     model=RFopt$coords$new_coord_system)
+     return(ats)
     }
   } else if (missing(alternative) || (class(alternative) %in% classes))
     stop("alternative model is not given or not of model type")
 
-  
-  if (!is.null(seed) && !is.na(seed)) set.seed(seed)
-  else if (!is.na(RFopt$general$seed)) {
-    if (printlevel >= PL.IMPORPANT)
-      message("NOTE: 'RFratiotest' is performed with fixed random seed ",
-              RFopt$general$seed,
-              ".\nSet RFoptions(seed=NA) to make the seed arbitrary.")
-    set.seed(RFopt$general$seed)
+  if (exists(".Random.seed")) {
+    old.seed <- .Random.seed
+    on.exit(.Random.seed <<- old.seed, add = TRUE)
   }
 
-  nullmodel <- PrepareModel2(nullmodel)
-  alternative <- PrepareModel2(alternative)
+  if (!is.null(seed) && !is.na(seed)) set.seed(seed)
+  else if (!is.na(RFopt$basic$seed)) {
+    if (printlevel >= PL_IMPORTANT)
+      message("NOTE: 'RFratiotest' is performed with fixed random seed ",
+              RFopt$basic$seed,
+              ".\nSet RFoptions(seed=NA) to make the seed arbitrary.")
+    set.seed(RFopt$basic$seed)
+  }
+
+  nullmodel <- PrepareModel2(nullmodel, ...)
+  alternative <- PrepareModel2(alternative, ...)
 
   Z <- StandardizeData(x=x, y=y, z=z, T=T, grid=grid, data=data,
-                       distances=distances, dimensions=dim, RFopt=RFopt)
+                       distances=distances, dim=dim, RFopt=RFopt)
   values <- try(GetValuesAtNA(NAmodel=nullmodel, valuemodel=alternative,
-                              spdim=Z$spdim, Time=Z$tsdim-Z$spdim,
+                              spatialdim=Z$spatialdim, Time=Z$Zeit,
                               shortnamelength=3, skipchecks=FALSE),
                 silent=TRUE)
   remove("Z")
 
   isSubmodel <- is.numeric(values) && all(is.na(values))
-  if (!isSubmodel && printlevel >= PL.IMPORPANT)
+  if (!isSubmodel && printlevel >= PL_IMPORTANT)
     message("'nullmodel' cannot be automatically detected as being a nullmodel of 'alternative'")
   
   model.list <- list(nullmodel=nullmodel, alternative=alternative)
@@ -359,7 +390,7 @@ RFratiotest <-
   for (m in 1:length(model.list)) {
     data.fit[[m]] <-
       RFfit(model.list[[m]], x=x, y=y, z=z, T=T, grid=grid, data=data,
-            lower=lower, upper=upper, bc_lambda=bc_lambda,
+            lower=lower, upper=upper,
              methods=methods,
             sub.methods=sub.methods, optim.control=optim.control,
             users.guess=guess,
@@ -373,41 +404,45 @@ RFratiotest <-
   if (RFopt$fit$ratiotest_approx) {
     ats <- approx_test(data.fit)
     class(ats) <- "RFratiotest"
-    return(ats)
+    if (RFopt$general$returncall)
+      attr(ats, "call") <- as.character(deparse(match.call()))
+    attr(ats, "coord_system") <- c(orig=RFopt$coords$coord_system,
+                                   model=RFopt$coords$new_coord_system)
+   return(ats)
   }
   
   model <- data.fit[[1]]$ml$model
-  data.ratio <- -diff(sapply(data.fit, function(x) x$ml$ml.value))
+  data.ratio <- -diff(sapply(data.fit, function(x) x$ml$likelihood))
   stopifnot(!isSubmodel || data.ratio <= 0) # should never appear
 
   simu.n <- n - 1
   ratio <- numeric(simu.n)
   fit <- numeric(2)
   Z <- StandardizeData(x=x, y=y, z=z, T=T, grid=grid, data=data,
-                       distances=distances, dimensions=dim, RFopt=RFopt)
-  dist.given <- !is.null(distances)
-  if (length(Z$neu) > 1) stop("multisets of data cannot be considered yet")
+                       distances=distances, dim=dim, RFopt=RFopt)
+  dist.given <- Z$dist.given
+  if (length(Z$coord) > 1) stop("multisets of data cannot be considered yet")
 
-  newx <- if (!dist.given) Z$neu[[1]]$x # lapply(Z$neu, function(x) x$x)
-  newT <- if (!dist.given) Z$neu[[1]]$T # lapply(Z$neu, function(x) x$T),
+  newx <- if (!dist.given) Z$coord[[1]]$x # lapply(Z$coord, function(x) x$x)
+  newT <- if (!dist.given) Z$coord[[1]]$T # lapply(Z$coord, function(x) x$T),
   
   pch <- if (RFopt$general$pch=="") "" else '@'
   for (i in 1:simu.n) {
-    if (printlevel>=PL.SUBIMPORPANT)
+    if (printlevel>=PL_SUBIMPORTANT)
       cat("\n ", i, "th simulation out of", simu.n)
     else cat(pch)
     simu <- RFsimulate(model, x=newx, T=newT, grid=grid, 
-                       distances=Z$distances[[1]], dim=dim, spC=FALSE)
+                       distances=if (dist.given) Z$coord[[1]], dim=dim, spC=FALSE)
     guess <- users.guess   
     for (m in 1:length(model.list)) {
       simufit <-
         RFfit(model.list[[m]], x=newx, T=newT, grid=grid,
               data=simu,
-              lower=lower, upper=upper, bc_lambda=bc_lambda,
+              lower=lower, upper=upper, 
               methods=methods,
               sub.methods=sub.methods, optim.control=optim.control,
               users.guess=guess,
-              distances=Z$distances[[1]], dim=dim,
+              distances=if (dist.given) Z$coord[[1]], dim=dim,
               transform=transform,
               ..., spConform=FALSE)
       fit[m] <- simufit$ml$ml
@@ -418,7 +453,7 @@ RFratiotest <-
 
     stopifnot(!isSubmodel || ratio[i] <= 0)# should never appear
     
-    if (printlevel > PL.SUBIMPORPANT)
+    if (printlevel > PL_SUBIMPORTANT)
       Print(c(data.ratio, ratio), fit, rank(c(data.ratio, ratio))[1])#
   }
   
@@ -430,8 +465,13 @@ RFratiotest <-
     paste("\nThe likehood ratio test ranks the likelihood of the data on rank",
           r, "among", simu.n, "simulations:", mess(alpha=alpha, p=p))
 
-  ret <- list(p=p, n=n, data.ratio=data.ratio, simu.ratios=ratio,
+  res <- list(p=p, n=n, data.ratio=data.ratio, simu.ratios=ratio,
               data.fit=data.fit, msg=msg, model.list=model.list)
-  class(ret) <- "RFratiotest"
+  class(res) <- "RFratiotest"
+  if (RFopt$general$returncall)
+    attr(res, "call") <- as.character(deparse(match.call()))
+  attr(res, "coord_system") <- c(orig=RFopt$coords$coord_system,
+                                 model=RFopt$coords$new_coord_system)
+  return(res)
 
 }
