@@ -4,7 +4,7 @@
 
  Gneiting's space-time covariance models and related models
 
- Copyright (C) 2006 -- 2014 Martin Schlather
+ Copyright (C) 2006 -- 2015 Martin Schlather
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -23,22 +23,18 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 
 #include <math.h>
- 
-#include "RF.h"
-#include "Covariance.h"
 #include <R_ext/Lapack.h>
-#include <R_ext/Linpack.h>
+#include <R_ext/Linpack.h> 
+#include "RF.h"
+#include "Operator.h"
 
 
-
-// #define AVERAGE_YPHASE 0
-// #define AVERAGE_YFREQ 1
 #define AVESTP_MINEIGEN 2
 #define AVESTP_LOGDET 3
 #define AVESTP_V 4
 #define AVESTP_LOGV 5
 #define AVESTP_LOGMIXDENS 6 
-#define TOTALAVESTP AVESTP_LOGMIXDENS + 1
+#define TOTALAVESTP (AVESTP_LOGMIXDENS + 1)
 
 #define AVE_A 0
 #define AVE_Z 1
@@ -63,7 +59,6 @@ void ave(double *h, cov_model *cov, double *v) {
   cov_model *next = cov->sub[0];
   int i,j,k,d,
     dim = spacetime ? cov->tsdim - 1 : cov->tsdim;
-    // dimP1 = dim + 1;
   double  detEplus2B, Ah[AveMaxDim], Eplus2B[AveMaxDim], 
     dummy, 
     hh,
@@ -126,12 +121,11 @@ int checkave(cov_model *cov) {
 	warning("A is not symmetric -- lower part used");
       }
   // naechste Zeile stimmt nicht mit Bernoulli ueberein
-  // if (!is_positive_definite(A, spdim)) SERR("A is not positive definite");
 
   kdefault(cov, AVE_SPACETIME, TRUE);
   if ((err = checkkappas(cov)) != NOERROR) return err;
 
-  if (cov->xdimprev != cov->tsdim || cov->xdimprev != cov->tsdim)
+  if (cov->xdimprev != cov->tsdim) // || cov->xdimprev != cov->tsdim)
     return ERRORDIM;
   if ((err = CHECK(next, dim, 1, PosDefType, XONLY, ISOTROPIC,
 		   SCALAR, cov->role // ROLE_COV changed 20.7.14 wg spectral
@@ -191,7 +185,7 @@ void sd_avestp(cov_model *cov, gen_storage VARIABLE_IS_NOT_USED *S, int dim, dou
   }
   // x2 *= 0.25;
   b = 3.0 * q[AVESTP_V] * x2 / dim;
-  alphamin = (4.0 + 4.0 * b - 2.0 * sqrt(4 * b * b + 8.0 * b + 1.0)) / 3.0;
+  alphamin = (4.0 + 4.0 * b - 2.0 * sqrt(4.0 * b * b + 8.0 * b + 1.0)) / 3.0;
   InvSqrt2a = 1.0 / sqrt(2.0 * alphamin * 6.0 * q[AVESTP_V]);
   *sd = InvSqrt2a;
   EmA = 1.0 - alphamin;
@@ -209,34 +203,32 @@ void sd_avestp(cov_model *cov, gen_storage VARIABLE_IS_NOT_USED *S, int dim, dou
   assert(radius > 0);
   */
 
-  // if (cov->mpp.refradius<0 || radius < cov->mpp.refradius)
-  //  cov->mpp.refradius = radius;
 }
 
 
 int structAve(cov_model *cov, cov_model **newmodel) { 
-  cov_model *shape, *gauss;
+  cov_model *shape, *gaussmix;
   int err;
  
   ASSERT_NEWMODEL_NOT_NULL;
   
   if (cov->role != Average) ILLEGAL_ROLE;
 
-  if ((err = covcpy(newmodel, cov)) != NOERROR) return err;
+  if ((err = covCpy(newmodel, cov)) != NOERROR) return err;
   shape = *newmodel;
   shape->nr = SHAPEAVE;
   addModel(shape, AVE_GAUSS, GAUSS);
-  gauss = shape->sub[AVE_GAUSS];
-  gauss->tsdim = 1;
-  gauss->role = ROLE_GAUSS;
-  gauss->method = SpectralTBM;
+  gaussmix = shape->sub[AVE_GAUSS];
+  gaussmix->tsdim = 1;
+  gaussmix->role = ROLE_GAUSS;
+  gaussmix->method = SpectralTBM;
   return NOERROR;
 }
 
 
 
 
-void  logshapeave(double *x, cov_model *cov, double *v, double *sign) {
+void  logshapeave(double *x, cov_model *cov, double *v, double *Sign) {
     // nur stationaer
   bool 
     spacetime = (bool) (PisNULL(AVE_SPACETIME) || P0INT(AVE_SPACETIME));
@@ -264,9 +256,9 @@ void  logshapeave(double *x, cov_model *cov, double *v, double *sign) {
     // zweiter Term fuer logg 
     //+ CovList[phi->nr].logmixdens(x, q[AVESTP_LOGV], phi); /* g */// nicht gatternr
     ;
-  sign[0] = 1.0;
+  Sign[0] = 1.0;
   double phase = q[AVERAGE_YPHASE] + q[AVERAGE_YFREQ] * (f - t); // Y
-  sign[1] =  phase > 0.0 ? 1.0 : phase < 0.0 ? -1.0 : 0.0;
+  Sign[1] =  phase > 0.0 ? 1.0 : phase < 0.0 ? -1.0 : 0.0;
   v[1] = log(fabs(phase));
 }
 
@@ -280,7 +272,7 @@ int check_shapeave(cov_model *cov) {
 int init_shapeave(cov_model *cov, gen_storage *s) { 
   ASSERT_GAUSS_METHOD(Average);
   cov_model
-    *gauss = cov->sub[AVE_GAUSS];
+    *gaussmix = cov->sub[AVE_GAUSS];
   double sd,
     *q = cov->q;
   bool 
@@ -294,20 +286,17 @@ int init_shapeave(cov_model *cov, gen_storage *s) {
   q[AVESTP_LOGDET] = 0.0;
   sd_avestp(cov, s, dim, &sd); // sd->gauss
 
-  assert(cov->vdim2[0] == 1);  assert(cov->vdim2[0] == cov->vdim2[1]);
+  assert(cov->vdim[0] == 1);  assert(cov->vdim[0] == cov->vdim[1]);
 
   if (cov->mpp.moments >= 0) {
     cov->mpp.mM[0] = cov->mpp.mMplus[0] = 1.0; 
     if (cov->mpp.moments >= 1) {
-      if ((err = INIT(gauss, cov->mpp.moments, s)) != NOERROR) return err;
+      if ((err = INIT(gaussmix, cov->mpp.moments, s)) != NOERROR) return err;
       if (cov->mpp.moments >= 2) {
 	cov->mpp.mM[2] = 1.0;
       }
     }
   }
-  //cov->mpp.loc_done = true;
-  //cov->mpp.refsd = sd;
-
   return err;
 }
 
@@ -317,7 +306,7 @@ void do_shapeave(cov_model *cov, gen_storage *S) {
    cov_model    
     *aveGAUSS = cov->sub[AVE_GAUSS],
     *phi = cov->sub[AVE_PHI];
-  double spectral[StpMaxDim], sd,
+  double spec_ret[StpMaxDim], sd,
     *q = cov->q;
   bool 
     spacetime = (bool) (PisNULL(AVE_SPACETIME) || P0INT(AVE_SPACETIME));
@@ -329,13 +318,12 @@ void do_shapeave(cov_model *cov, gen_storage *S) {
   
   BUG;
 
-  SPECTRAL(aveGAUSS, S, spectral);  // nicht gatternr
-  q[AVERAGE_YFREQ] = *spectral * q[AVESTP_V];  
+  SPECTRAL(aveGAUSS, S, spec_ret);  // nicht gatternr
+  q[AVERAGE_YFREQ] = *spec_ret * q[AVESTP_V];  
   q[AVERAGE_YPHASE] = TWOPI * UNIFORM_RANDOM;
 
   
   BUG; // what to do with the next line?
-  // info->logdens = CovList[phi->nr].logmixdens(ZERO, q[AVESTP_LOGV], phi);
  }
 
 
@@ -356,27 +344,25 @@ void do_shapeave(cov_model *cov, gen_storage *S) {
 void GetEu2Dinv(cov_model *cov, double *x, int dim, 
 		double *det, double *Eu2Dinv,
 		double *newxsq, double *newx, double *z) {
-    double t, t2,
+    double 
 	y[CoxMaxDim],
       *V = P(COX_MU),
       *D= P(COX_D),
-      beta = P0(COX_BETA);
+      beta = P0(COX_BETA),
+      t = x[dim],
+      t2 = pow(fabs(t), beta); // standard t^2
     int d,
 	dimP1 = dim + 1,
 	dimsq = dim * dim;
-  t = x[dim];
-  t2 = pow(fabs(t), beta); // standard t^2
-  for (d=0; d<dim; d++) {
+    for (d=0; d<dim; d++) {
       y[d] = x[d] - t * V[d];
   }
   
   for (d=0; d<dimsq; d++) {
-    // printf("%d %d\n", d, dimsq);
       Eu2Dinv[d] = t2 * D[d];
   }
   for (d=0; d<dimsq; d+=dimP1)  Eu2Dinv[d] += 1.0; // D + E
   det_UpperInv(Eu2Dinv, det, dim);
-//  print("t=%f, %f %f\n", t, t2, *det);
   *newxsq = xUxz(y, Eu2Dinv, dim, z);
   *newx = sqrt(*newxsq);
 }
@@ -434,9 +420,6 @@ void cox(double *x, cov_model *cov, double *v) {
   int dim = cov->tsdim - 1,
       dimsq = dim * dim;
   double det, newx,  newxsq;
- 
-  //PMI(cov, "cox");
-
   ALLOC_EXTRA(Eu2Dinv, dimsq);
   GetEu2Dinv(cov, x, dim, &det, Eu2Dinv, &newxsq, &newx, NULL);
    
@@ -498,14 +481,10 @@ int checkcox(cov_model *cov) {
   int err, i, 
     dim = cov->tsdim - 1,
     dimsq = dim * dim; 
-  //  APMI(cov);
-  //printf("AAAAAAAAAAAA\n");
-
 
   if (cov->xdimown < 2) SERR("The space-time dimension must be at least 2.");
    
   if (cov->ncol[COX_MU] != 1 || cov->nrow[COX_MU] != dim) {
-    //  print("%d %d %d\n", cov->nrow[COX_MU],  dim,cov->nrow[COX_MU] != dim);
     if (cov->ncol[COX_MU] == dim && cov->nrow[COX_MU] == 1) {
       cov->nrow[COX_MU] = dim;
       cov->ncol[COX_MU] = 1; 
@@ -573,7 +552,6 @@ void rangecox(cov_model VARIABLE_IS_NOT_USED *cov, range_type* range){
 }
 
 int initcox(cov_model *cov, gen_storage *s) {
-  // PMI(cov);
   ASSERT_GAUSS_METHOD(SpectralTBM);
   cov_model *next = cov->sub[0];
   return INIT(next, 0, s);
@@ -589,7 +567,7 @@ void spectralcox(cov_model *cov, gen_storage *s, double *e) {
   SPECTRAL(next, s, e); // nicht gatternr
   
   v[0] = rnorm(0.0, INVSQRTTWO);
-  v[1] = rho * v[0] + sqrt(1 - rho * rho) * rnorm(0.0, INVSQRTTWO);
+  v[1] = rho * v[0] + sqrt(1.0 - rho * rho) * rnorm(0.0, INVSQRTTWO);
  
   for (t = 0.0, d=0; d<dim; d++) {
     t += (v[d] + V[d]) * e[d];
@@ -603,13 +581,9 @@ void spectralcox(cov_model *cov, gen_storage *s, double *e) {
 // GenPS (generalisation of paciore and stein)
 // Q = (x-y) Sy (Sx + Sy)^{-1} Sx (x-y) (weicht etwas von Stein ab)
 
-#define STP_S 0
-#define STP_Z 1
-#define STP_M 2
 
 #define STP_XI2 0
 #define STP_PHI 1
-//#define STP_SX 2
 #define STP_GAUSS 3
 void kappa_stp(int i, cov_model *cov, int *nr, int *nc){
   *nc = (i == STP_S || i == STP_M) ? cov->tsdim : 1;
@@ -624,7 +598,6 @@ void stp(double *x,  double *y, cov_model *cov, double *v) {
     Mh[StpMaxDim], hSx[StpMaxDim],
     Syh[StpMaxDim], xi2x, xi2y, 
     detA, hMh, cxy, zh, Q, Amux[StpMaxDim], Amuy[StpMaxDim],
-    // Q2, Q3, 
     *Sc = P(STP_S),
     *M = P(STP_M),
     *z = P(STP_Z);
@@ -648,7 +621,6 @@ void stp(double *x,  double *y, cov_model *cov, double *v) {
     //      for (ii=0; ii<4; ii++) printf("%f ", Sy[ii]);
     //      printf("\n");
     //    }
-    //APMI(Sf);
 
   } else {
     int bytes = sizeof(double) * dimsq;
@@ -665,10 +637,8 @@ void stp(double *x,  double *y, cov_model *cov, double *v) {
 
   for (k=0, d=0; d<dim; d++) {
     h[d] = x[d] - y[d];
-    // print("%d: %f %f\n", d, x[d], y[d]);
   } 
 
-  //Q2 = Q3 = 
   zh = hMh = 0.0;
   for (k=0, d=0; d<dim; d++) {
     Mh[d] = hSx[d] = Syh[d] = 0.0;
@@ -679,21 +649,12 @@ void stp(double *x,  double *y, cov_model *cov, double *v) {
     }
     zh += z[d] * h[d];
     hMh += Mh[d] * h[d];
-    //Q2 += Syh[d] * h[d];
-    //Q3 += hSx[d] * h[d];
-   }
+  }
   cxy = xi2x - xi2y - zh;
-  //Q2 += (hMh - cxy) * (hMh - cxy); 
-  //Q3 += (hMh + cxy) * (hMh + cxy); 
-
-  // print("%f %f\N", 
-
-  //printf("x=%f %f y=%f %f\n", x[0], x[1], y[0], y[1]);
   
   for (k=d=0; d<dim; d++) {
     for (j=0; j<dim; j++, k++) {
       A[k] = Sx[k] + Sy[k] + 4.0 * Mh[d] * Mh[j];
-      //printf("d=%d %d S=%f %f M=%f %f A=%f\n", d, j, Sx[k], Sy[k], Mh[d], Mh[j], A[k]);
       assert(R_FINITE(A[k]));
     }
     Amux[d] = hSx[d] + 2.0 * (hMh + cxy) * Mh[d]; // uses that M is symmetric
@@ -735,9 +696,12 @@ int checkstp(cov_model *cov){
     *phi = cov->sub[STP_PHI],
     *Sf = cov->kappasub[STP_S],
     *xi2 =cov->sub[STP_XI2];
-  int err;
-  
-  int dim = cov->tsdim;
+  int err,
+    dim = cov->tsdim;
+
+
+  ASSERT_CARTESIAN;
+
  if (dim > StpMaxDim)
     SERR2("For technical reasons max. dimension for ave is %d. Got %d.", 
 	  StpMaxDim, cov->xdimprev);
@@ -756,7 +720,7 @@ int checkstp(cov_model *cov){
    PALLOC(STP_Z, dim, 1);
  }
 
- if (cov->xdimprev != cov->tsdim || cov->xdimprev != cov->tsdim)
+ if (cov->xdimprev != cov->tsdim) // || cov->xdimprev != cov->tsdim)
     return ERRORDIM;
    
   if ((err = CHECK(phi, dim,  1, PosDefType, XONLY, ISOTROPIC,
@@ -812,7 +776,7 @@ int structStp(cov_model *cov, cov_model **newmodel) {
   
   if (cov->role != Average) ILLEGAL_ROLE;
   
-  if ((err = covcpy(newmodel, cov)) != NOERROR) return err;
+  if ((err = covCpy(newmodel, cov)) != NOERROR) return err;
   shape = *newmodel;
   shape->nr = SHAPESTP;
   addModel(shape, STP_GAUSS, GAUSS);
@@ -834,7 +798,7 @@ int init_shapestp(cov_model *cov, gen_storage *s) {
 
   cov_model
     *Sf = cov->kappasub[STP_S],
-    *gauss = cov->sub[STP_GAUSS];
+    *gaussmix = cov->sub[STP_GAUSS];
   double sd,
     *q = cov->q;
   int err = NOERROR;
@@ -843,11 +807,11 @@ int init_shapestp(cov_model *cov, gen_storage *s) {
     double minmax[2];
     assert(CovList[Sf->nr].minmaxeigenvalue != NULL);
     CovList[Sf->nr].minmaxeigenvalue(Sf, minmax);
-    if (minmax[0] <= 0.0) error("neg eigenvalue in shape function of 'stp'");
+    if (minmax[0] <= 0.0) ERR("neg eigenvalue in shape function of 'stp'");
     q[AVESTP_MINEIGEN] = minmax[0];
     q[AVESTP_LOGDET] = (double) cov->xdimprev * log(minmax[1]);
   } else {
-#define dummyN 5 * StpMaxDim
+#define dummyN (5 * StpMaxDim)
     double value[StpMaxDim], ivalue[StpMaxDim], dummy[dummyN], det,
       min = RF_INF;
     int i, Ferr,       
@@ -863,7 +827,6 @@ int init_shapestp(cov_model *cov, gen_storage *s) {
       double v = fabs(value[i]);
       det *= v;
       if (min > v) min = v;
-      //if (max < value[i]) max = v;
     }
     q[AVESTP_MINEIGEN] = min;
     q[AVESTP_LOGDET] = log(det);
@@ -874,19 +837,15 @@ int init_shapestp(cov_model *cov, gen_storage *s) {
   q[AVESTP_LOGMIXDENS] = 0.0;
   sd_avestp(cov, s, cov->tsdim, &sd); // sd->gauss
 
-  assert(cov->vdim2[0] == 1);  assert(cov->vdim2[0] == cov->vdim2[1]);
+  assert(cov->vdim[0] == 1);  assert(cov->vdim[0] == cov->vdim[1]);
 
   if (cov->mpp.moments >= 0) {
     cov->mpp.mM[0] = cov->mpp.mMplus[0] = 1.0; //// ??? notwendig 
     if (cov->mpp.moments >= 1) {
-      if ((err = INIT(gauss, 2, s)) != NOERROR) return err;
+      if ((err = INIT(gaussmix, 2, s)) != NOERROR) return err;
       if (cov->mpp.moments >= 2) cov->mpp.mM[2] = 1.0; 
     }
   }
-  
-  //cov->mpp.loc_done = true;
-  //cov->mpp.refsd = sd;
-
   return err;
 }
 
@@ -895,7 +854,7 @@ void do_shapestp(cov_model *cov, gen_storage *s) {
   cov_model 
     *stpGAUSS = cov->sub[STP_GAUSS],
     *phi = cov->sub[STP_PHI];
-  double  spectral[StpMaxDim], sd,
+  double  spec_ret[StpMaxDim], sd,
     *q = cov->q;
 
   CovList[phi->nr].drawmix(phi, &(q[AVESTP_V]));
@@ -903,8 +862,8 @@ void do_shapestp(cov_model *cov, gen_storage *s) {
   
   BUG;
 
-  SPECTRAL(stpGAUSS, s, spectral);  // nicht gatternr
-  q[AVERAGE_YFREQ] = *spectral * sqrt(q[AVESTP_V]);  
+  SPECTRAL(stpGAUSS, s, spec_ret);  // nicht gatternr
+  q[AVERAGE_YFREQ] = *spec_ret * sqrt(q[AVESTP_V]);  
   q[AVERAGE_YPHASE] = TWOPI * UNIFORM_RANDOM;
 
 
@@ -917,7 +876,7 @@ void do_shapestp(cov_model *cov, gen_storage *s) {
 }
 
 
-void logshapestp(double *x, double *u, cov_model *cov, double *v, double *sign){
+void logshapestp(double *x, double *u, cov_model *cov, double *v, double *Sign){
   // kann um ca. Faktor 2 beschleunigt werden, wenn
   // Sx , logdetU, Hx fuer alle x abgespeichert werden
   // und die Werte dann aus dem Speicher gelesen werden
@@ -992,7 +951,7 @@ void logshapestp(double *x, double *u, cov_model *cov, double *v, double *sign){
  
   double cos_value = cos(q[AVERAGE_YPHASE] + q[AVERAGE_YFREQ] * xi);
   *v = exponent + log(fabs(cos_value)) ;  // Y 
-  *sign = cos_value > 0.0 ? 1.0 : cos_value < 0.0 ? -1.0 : 0.0;
+  *Sign = cos_value > 0.0 ? 1.0 : cos_value < 0.0 ? -1.0 : 0.0;
 }
 
 
@@ -1037,13 +996,10 @@ void rational(double *x, cov_model *cov, double *v) {
 }
  
 int checkrational(cov_model *cov){
-//  pref_type pref = 
-//    {5, 0, 0, 0, 0, 5, 5, 0, 0, 0, 0, 0, 5};
-  // CE CO CI TBM Sp di sq Ma av n mpp Hy any
   int err;
   if (cov->nrow[RATIONAL_a] == 1) {
     double dummy = P0(RATIONAL_a);
-    free(P(RATIONAL_a));
+    PFREE(RATIONAL_a);
     PALLOC(RATIONAL_a, 2, 1);
     P(RATIONAL_a)[0] = dummy;
     P(RATIONAL_a)[1] = 0.0;
@@ -1109,7 +1065,7 @@ void minmaxEigenEAxxA(cov_model *cov, double *mm) {
     *E = P(EAXXA_E);
   int i,
     dim = cov->tsdim;
-  for (mm[0] = RF_INF, mm[1]=-RF_INF, i=0; i<dim; i++) {
+  for (mm[0] = RF_INF, mm[1]=RF_NEGINF, i=0; i<dim; i++) {
     if (E[i] < mm[0]) mm[0] = E[i];
     if (E[i] > mm[1]) mm[1] = E[i];
   }
@@ -1117,17 +1073,13 @@ void minmaxEigenEAxxA(cov_model *cov, double *mm) {
  
 int checkEAxxA(cov_model *cov){
   int err;
-  //  pref_type pref = 
-  //    {5, 0, 0, 0, 0, 5, 5, 0, 0, 0, 0, 0, 5};
-  // CE CO CI TBM Sp di sq Ma av n mpp Hy any
-  //  MEMCOPY(cov->pref, pref, sizeof(pref_type));  
     if (cov->xdimown > EaxxaMaxDim)
       SERR2("For technical reasons max. dimension for ave is %d. Got %d.", 
 	    EaxxaMaxDim, cov->xdimown);
     
   if ((err = checkkappas(cov)) != NOERROR) return err;
 
-  cov->vdim2[0] = cov->vdim2[1] = cov->tsdim;
+  cov->vdim[0] = cov->vdim[1] = cov->tsdim;
   cov->mpp.maxheights[0] = RF_NA;
  return NOERROR;
 }
@@ -1155,7 +1107,7 @@ void rangeEAxxA(cov_model VARIABLE_IS_NOT_USED *cov, range_type* range){
 
 // Sigma(x) = diag>0 + A'xx'A
 void kappa_EtAxxA(int i, cov_model VARIABLE_IS_NOT_USED *cov, int *nr, int *nc){
-  int tsdim = 3; //  cov->tsdim
+  int tsdim = 3; 
   *nc = (i == EAXXA_A) ? tsdim : 1;
   *nr = (i == EAXXA_E || i==EAXXA_A) ? tsdim : (i==ETAXXA_ALPHA) ? 1 : -1;
 }
@@ -1212,7 +1164,7 @@ void minmaxEigenEtAxxA(cov_model *cov, double *mm) {
     *E = P(EAXXA_E);
   int i,
     dim = cov->tsdim;
-  for (mm[0] = RF_INF, mm[1]=-RF_INF, i=0; i<dim; i++) {
+  for (mm[0] = RF_INF, mm[1]=RF_NEGINF, i=0; i<dim; i++) {
     if (E[i] < mm[0]) mm[0] = E[i];
     if (E[i] > mm[1]) mm[1] = E[i];
   }
@@ -1220,12 +1172,8 @@ void minmaxEigenEtAxxA(cov_model *cov, double *mm) {
  
 int checkEtAxxA(cov_model *cov){
   int err;
-//  pref_type pref = 
-//    {5, 0, 0, 0, 0, 5, 5, 0, 0, 0, 0, 0, 5};
-  // CE CO CI TBM Sp di sq Ma av n mpp Hy any
-//  MEMCOPY(cov->pref, pref, sizeof(pref_type));  
   if (cov->xdimown != 3) SERR("The space-time dimension must be 3.");
-  cov->vdim2[0] = cov->vdim2[1] = cov->tsdim;
+  cov->vdim[0] = cov->vdim[1] = cov->tsdim;
   if ((err = checkkappas(cov)) != NOERROR) return err;
   cov->mpp.maxheights[0] = RF_NA;
  return NOERROR;
@@ -1271,7 +1219,6 @@ void rotat(double *x, cov_model *cov, double *v) {
     absx = sqrt(x[0] * x[0] + x[1] * x[1]);
   *v = (absx == 0.0) ? 0.0
     : speed * (cos(phi * x[time]) * x[0] + sin(phi * x[time]) * x[1]) / absx;
-  // print("%f\n", *v);
 }
 
 void minmaxEigenrotat(cov_model VARIABLE_IS_NOT_USED *cov, double *mm) {
@@ -1281,11 +1228,6 @@ void minmaxEigenrotat(cov_model VARIABLE_IS_NOT_USED *cov, double *mm) {
  
 int checkrotat(cov_model *cov){
   int err;
-//  if (cov->tsdim != 3) return("only 3-d allowed for rotat!");
-//  pref_type pref = 
-//    {5, 0, 0, 0, 0, 5, 5, 0, 0, 0, 0, 0, 5};
-  // CE CO CI TBM Sp di sq Ma av n mpp Hy any
-//  MEMCOPY(cov->pref, pref, sizeof(pref_type));  
   if (cov->xdimown != 3) SERR("The space-time dimension must be 3.");
    if ((err = checkkappas(cov)) != NOERROR) return err;
  cov->mpp.maxheights[0] = RF_NA;
@@ -1338,7 +1280,7 @@ int checkRotat(cov_model *cov){
   int err;
   if (cov->xdimown != 3) SERR("The space-time dimension must be 3.");
   if ((err = checkkappas(cov)) != NOERROR) return err;
-  cov->vdim2[0] = cov->vdim2[1] = cov->tsdim;
+  cov->vdim[0] = cov->vdim[1] = cov->tsdim;
   cov->mpp.maxheights[0] = RF_NA;
   return NOERROR;
 }
@@ -1354,233 +1296,47 @@ void rangeRotat(cov_model VARIABLE_IS_NOT_USED *cov, range_type* range){
 }
 
 
-
-/* Whittle-Matern or Whittle or Besset */ 
-// statt 0 Parameter: 2 Parameter, M und z fuer xi
-void kappaNonStWM(int i, cov_model *cov, int *nr, int *nc){
-  *nc =  1;
-  *nr = i < CovList[cov->nr].kappas ? 1 : -1;
-}
-
-void NonStWMQ(double *x, double *y, double sqrtQ, cov_model *cov, double *v){
-// check calling functions, like hyperbolic and gneiting if any changings !!
-  double loggamma, nuxy, nux, nuy;
-  cov_model *nu = cov->kappasub[WM_NU];
-
-  if (nu == NULL) {
-    nuxy = P0(WM_NU);
-    loggamma = lgammafn(nuxy);
-  } else {
-    FCTN(x, nu, &nux);
-    FCTN(y, nu, &nuy);
-    nuxy = 0.5 * (nux + nuy);
-    loggamma = 0.5 * (lgammafn(nux) + lgammafn(nuy));
-  }
-    
-  if (sqrtQ == 0.0) {
-    *v = 1.0;
-    return;
-  }
- 
-  *v = 2.0 * exp(nuxy * log(0.5 * sqrtQ) - loggamma
-		 + log(bessel_k(sqrtQ, nuxy, 2.0)) - sqrtQ);
-}
-
-void NonStWM(double *x, double *y, cov_model *cov, double *v){
-// check calling functions, like hyperbolic and gneiting if any changings !!
-  int d,
-    dim = cov->tsdim;
-  double Q=0.0;
-
-  //  assert(false);
-
-  for (d=0; d<dim; d++) {
-    double delta = x[d] - y[d];
-    Q += delta * delta;
-  }
-  NonStWMQ(x, y, sqrt(Q), cov, v);
-}
-
-int checkNonStWM(cov_model *cov) { 
-  cov_model *nu = cov->kappasub[WM_NU];
-  int err,
-    dim = cov->tsdim;
-
-  NotProgrammedYet("");
- 
-
-  if (PisNULL(WM_NU) && nu==NULL) SERR("'nu' is missing");
-  if (isRandom(CovList[cov->nr].kappaParamType[WM_NU])) 
-    SERR("only deterministic models for 'nu' are allowed.\nHowever these models can have random parameters.");
-  
-  if (nu != NULL) {
-    if ((err = CHECK(nu, dim, dim, ShapeType, XONLY, CARTESIAN_COORD,
-		       SCALAR, cov->role // ROLE_COV changed 20.7.14 wg spectral
-		     )) != NOERROR) 
-      return err;
-    if (nu->tsdim != cov->tsdim) return ERRORWRONGDIM;
-  }
-  
-  //PMI(cov);
-
-  // no setbackard !!
-  return NOERROR;
-}
-
-sortsofparam paramtype_nonstWM(int VARIABLE_IS_NOT_USED  k, int VARIABLE_IS_NOT_USED  row, int VARIABLE_IS_NOT_USED col) {
-  return CRITICALPARAM;
-}
-
-
-void rangeNonStWM(cov_model VARIABLE_IS_NOT_USED *cov, range_type* range){ 
-  range->min[WM_NU] = 0.0;
-  range->max[WM_NU] = RF_INF;
-  range->pmin[WM_NU] = 1e-2;
-  range->pmax[WM_NU] = 10.0;
-  range->openmin[WM_NU] = true;
-  range->openmax[WM_NU] = true;
-}
-
-// using nu^(-1-nu+a)/2 for g and v^-a e^{-1/4v} as density instead of frechet
-// the bound 1/3 can be dropped
-// static double eM025 = exp(-0.25);
-//void DrawMixNonStWM(cov_model *cov, double *random) { // inv scale
-//  // V ~ F in stp
-//  cov_model *nu = cov->sub[WM_NU];  
-//  double minnu;
-//  double alpha;
-//
-//  if (nu == NULL) {
-//    minnu = P(WM_NU][0];
-//  } else {
-//    double minmax[2];
-//    CovList[nu->nr].minmaxeigenvalue(nu, minmax);
-//    minnu = minmax[0];
-//  }
-//  alpha = 1.0 + 0.5 /* 0< . < 1*/ * (3.0 * minnu - 0.5 * cov->tsdim);
-//  if (alpha > 2.0) alpha = 2.0; // original choice
-//  if (alpha <= 1.0) ERR("minimal nu too low or dimension too high");
-//
-// error("logmixdensnonstwm not programmed yet");
- /* 
-  double beta = GLOBAL.mpp.beta,
-    p = GLOBAL.mpp.p,
-    logU;
-  if (UNIFORM_RANDOM < p){
-    cov_a->WMalpha = beta;
-    logU =  log(UNIFORM_RANDOM * eM025);
-    cov_a->WMfactor = -0.5 * log(0.25 * p * (beta - 1.0)) + 0.25;
-  } else {
-    cov_a->WMalpha = alpha;
-    logU = log(eM025 + UNIFORM_RANDOM * (1.0 - eM025));
-    cov_a->WMfactor = -0.5 * log(0.25 * (1.0 - p) * (alpha - 1.0));
-  } 
-  
-  logmix!!
-
-  *random = log(-0.25 / logU) / (cov_a->WMalpha - 1.0); //=invscale
-  */
-//}
-
-
-
-
-//double LogMixDensNonStWM(double *x, double logV, cov_model *cov) {
-//  // g(v,x) in stp
-//  double z = 0.0;
-//  error("logmixdensnonstwm not programmed yet");
-  // wmfactor ist kompletter unsinn; die 2 Teildichten muessen addiert werden
-  /*
-  cov_model *calling = cov->calling,
-    *Nu = cov->sub[0];
-  if (calling == NULL) BUG;
-   double nu,
-    alpha = cov_a->WMalpha,
-    logV = cov_a->logV,
-    V = cov_a->V;
-  
-  if (Nu == NULL) 
-    nu = P(WM_NU][0];
-  else 
-     FCTN(x, Nu, &nu);
-
-
-   z = - nu  * M_LN2 // in g0  // eine 2 kuerzt sich raus
-    + 0.5 * ((1.0 - nu) // in g0
-    + alpha // lambda
-    - 2.0 //fre*
-    ) * logV
-    - 0.5 * lgammafn(nu)  // in g0
-    + cov_a->WMfactor // lambda
-    - 0.125 / V   // g: Frechet
-    + 0.125 * pow(V, 1.0 - alpha); // lambda: frechet
-
-  if (!(z < 7.0)) {
-    static double storage = 0.0; 
-    if (gen_storage != logV) {
-      if (PL >= PL_DETAILS) 
-	PRINTF("alpha=%f, is=%f, cnst=%f logi=%f lgam=%f loga=%f invlogs=%f pow=%f z=%f\n",
-	       alpha,V,
-	       (1.0 - nu) * M_LN2 
-	       , + ((1.0 - nu) * 0.5 + alpha - 2.0) * logV
-	       ,- 0.5 * lgammafn(nu) 
-	       , -cov_a->WMfactor
-	       ,- 0.25 / V 
-	       , + 0.25 * pow(V, - alpha)
-	       , z);
-      storage = logV;
-    }
-    //assert(z < 10.0);
-  }
-*/
-//  return z;
-//				      
-//}
-
-
-
-
 /* nsst */
 /* Tilmann Gneiting's space time models, part I */
-#define NSST_DELTA 0
 void nsst(double *x, cov_model *cov, double *v) {
   cov_model *subphi = cov->sub[0];
   cov_model *subpsi = cov->sub[1];
-  double v1, v2, psi, phi, y;
+  double v1, v2, psi, y;
   
   COV(ZERO, subpsi, &v1);
   COV(x + 1, subpsi, &v2);
   psi = sqrt(1.0 + v1 - v2);  // C0 : C(0) oder 0 // Cx : C(x) oder -gamma(x)
   y = x[0] / psi;
-  COV(&y, subphi, &phi);
-  *v = pow(psi, -P0(NSST_DELTA)) * phi;
-}
-
-void TBM2nsst(double *x, cov_model *cov, double *v) {
-  cov_model *subphi = cov->sub[0];
-  cov_model *subpsi = cov->sub[1];
-  double v1, v2, psi, phi, y;
-
-  COV(ZERO, subpsi, &v1);
-  COV(x + 1, subpsi, &v2);
-  psi = sqrt(1.0 + v1 - v2);  // C0 : C(0) oder 0 // Cx : C(x) oder -gamma(x)
-  y = x[0] / psi;
-  TBM2CALL(&y, subphi, &phi);
-  *v = pow(psi, -P0(NSST_DELTA)) * phi;
+  COV(&y, subphi, v);
+  *v *= pow(psi, -P0(NSST_DELTA));
 }
 
 void Dnsst(double *x, cov_model *cov, double *v) {
   cov_model *subphi = cov->sub[0];
   cov_model *subpsi = cov->sub[1];
-  double v1, v2, psi, phi, y;
+  double v1, v2, psi, y;
 
   COV(ZERO, subpsi, &v1);
   COV(x + 1, subpsi, &v2);
   psi = sqrt(1.0 + v1 - v2);  // C0 : C(0) oder 0 // Cx : C(x) oder -gamma(x)
   y = x[0] / psi;
-  Abl1(&y, subphi, &phi);
-  *v = pow(psi, -P0(NSST_DELTA) - 1) * phi;
-  // print("(%f %f %f)",  psi, y, *v);
+  Abl1(&y, subphi, v);
+  *v *= pow(psi, -P0(NSST_DELTA) - 1.0);
+}
+
+void TBM2nsst(double *x, cov_model *cov, double *v) {
+  cov_model *subphi = cov->sub[0];
+  cov_model *subpsi = cov->sub[1];
+  double v1, v2, psi, y;
+
+  assert(false);
+
+  COV(ZERO, subpsi, &v1);
+  COV(x + 1, subpsi, &v2);
+  psi = sqrt(1.0 + v1 - v2);  // C0 : C(0) oder 0 // Cx : C(x) oder -gamma(x)
+  y = x[0] / psi;
+  TBM2CALL(&y, subphi, v);
+  *v *= pow(psi, -P0(NSST_DELTA));
 }
 
 int checknsst(cov_model *cov) {
@@ -1601,25 +1357,19 @@ int checknsst(cov_model *cov) {
   setbackward(cov, subphi);
   assert(cov->finiterange != true); //
 
-  if ((err = CHECK(subpsi, 1, 1, NegDefType, XONLY, ISOTROPIC, 
+  if ((err = CHECK(subpsi, 1, 1, VariogramType, XONLY, ISOTROPIC, 
 		     SCALAR, cov->role // ROLE_COV changed 20.7.14 wg spectral
 		   )) != NOERROR) 
     return err;
 
-  subphi->delflag = subpsi->tsdim = DEL_COV-20;
+  subphi->delflag = subpsi->delflag = DEL_COV-20;
 
   // kein setbackward(cov, subpsi);
   return NOERROR;
 }
 
-sortsofparam paramtype_nsst(int k, int VARIABLE_IS_NOT_USED row, int VARIABLE_IS_NOT_USED col) {
-  return k==-1 ? VARPARAM : k==0 ? CRITICALPARAM : ANYPARAM;
-}
-
-void rangensst(cov_model *cov, range_type* range){
- 
-  //  print("dim min=%d\n",  cov->tsdim - 1);
 
+void rangensst(cov_model *cov, range_type* range){ 
   range->min[NSST_DELTA] = cov->tsdim - 1;
   range->max[NSST_DELTA] = RF_INF;
   range->pmin[NSST_DELTA] = cov->tsdim - 1;
@@ -1629,3 +1379,171 @@ void rangensst(cov_model *cov, range_type* range){
 }
 
 
+
+
+/* gennsst */
+/* generalisation in schlather, bernoulli 2010 */
+
+#define GENNSST_INTERN_A 0
+
+void kappa_gennsst_intern(int i, cov_model VARIABLE_IS_NOT_USED *cov, int *nr, int *nc){
+  *nc = *nr = i == 0 ? 0 : -1;
+}
+
+void nonstatgennsst_intern(double *x,double *y, cov_model *cov, double *v) {
+  cov_model *next = cov->sub[0];
+  double  det, z,
+    *A = P(GENNSST_INTERN_A);
+  int d, 
+    dim = cov->tsdim;
+  ALLOC_EXTRA(ds, dim);
+ 
+  //  printf("\nx=%f %f %f \ny=%f %f %f\nz=%f %f %f\n %f %f %f\n %f %f %f\n",  	 x[0], x[1], x[2], y[0],y[1],y[2],	 A[0],  A[1], A[2], A[3], A[4], A[5], A[6], A[7], A[8]);
+
+  //  PMI(cov->calling->sub[1]);
+
+  det_UpperInv(A, &det, dim);
+  for (d=0; d<dim; d++) ds[d] = x[d] - y[d];
+  z = sqrt(xUx(ds, A, dim));
+  COV(&z, next, v);
+  *v /= sqrt(det);
+}
+
+int checkgennsst_intern(cov_model *cov) {
+  // internes modell muss zwischengeschaltet werden
+  // um Earth->Cartesian->isotropic hinzubekommen
+  // und         ^- hier noch mit A zu multiplizieren
+  cov_model *next = cov->sub[0];
+  int err,
+    dim = cov->xdimown;
+
+  
+
+  assert(cov->tsdim == cov->xdimown);
+  if ((err = CHECK(next, cov->tsdim, 1, PosDefType, 
+		   XONLY, ISOTROPIC, SCALAR, cov->role )) != NOERROR) 
+    return err;
+  if (!isNormalMixture(next->monotone)) return ERRORNORMALMIXTURE;
+
+  //APMI(cov->calling);
+
+  if (PisNULL(GENNSST_INTERN_A)) {  
+    PALLOC(GENNSST_INTERN_A, dim, dim);
+  } else if (cov->nrow[GENNSST_INTERN_A] != dim) {
+    PFREE(GENNSST_INTERN_A);
+    PALLOC(GENNSST_INTERN_A, dim, dim);
+  }
+
+  cov->finiterange = false;
+  setbackward(cov, next);
+  assert(!cov->finiterange); 
+  cov->vdim[0] = cov->vdim[1] = 1;
+
+  EXTRA_STORAGE;
+  return NOERROR;
+}
+
+
+void range_gennsst_intern(cov_model VARIABLE_IS_NOT_USED *cov,
+			  range_type* range){
+  range->min[GENNSST_INTERN_A] = RF_NEGINF;
+  range->max[GENNSST_INTERN_A] = RF_INF;
+  range->pmin[GENNSST_INTERN_A] = -100.0;
+  range->pmax[GENNSST_INTERN_A] = +100.0;
+  range->openmin[GENNSST_INTERN_A] = true;
+  range->openmax[GENNSST_INTERN_A] = true;
+}
+
+
+#define GENNSST_DELTA 0
+void nonstatgennsst(double *x,double *y, cov_model *cov, double *v) {
+  cov_model *subphi = cov->key;
+  cov_model *subpsi = cov->sub[1];
+  int i, 
+    dim = cov->tsdim,
+    dimSq = dim * dim;
+    
+  //  print("A %s\n", TYPENAMES[subpsi->typus]);
+  NONSTATCOV(x, y, subpsi, PARAM(subphi, GENNSST_INTERN_A));
+  if (isVariogram(subpsi)) {
+    ALLOC_EXTRA(z, dimSq);
+    NONSTATCOV(ZERO, ZERO, subpsi, z);
+    for (i=0; i<dimSq; i++) 
+      PARAM(subphi, GENNSST_INTERN_A)[i] = 
+	z[i] - PARAM(subphi, GENNSST_INTERN_A)[i]; 
+  } else if (subpsi->typus != NegDefType) BUG;
+
+
+  //    print("%f %f %f %f  dim = %d %f %f %f %f %s\n", x[0], x[1], y[0], y[1], dim, v1[0], v1[1], v1[2], v1[3],NAME(subpsi));
+
+  NONSTATCOV(x, y, subphi, v);
+}
+  
+
+int checkgennsst(cov_model *cov) {
+  cov_model *subphi = cov->sub[0];
+  cov_model *subpsi = cov->sub[1];
+#define ncs 3
+  int err,
+    UCS = UpgradeToCoordinateSystem(cov->isoown);
+   assert(cov->tsdim == cov->xdimown);
+
+   if (cov->q == NULL) {
+     QALLOC(1);
+     cov->q[0]=NOERROR;
+   }
+
+   if (isSpherical(cov->isoown)) 
+     return cov->q[0]!=NOERROR ? (int) cov->q[0] : ERRORFAILED;
+   if (UCS == ISO_MISMATCH) 
+     return cov->q[0]!=NOERROR ? (int) cov->q[0] : ERRORODDCOORDTRAFO;
+   if (cov->xdimown != cov->tsdim) SERR("logical and physical dimension differ");
+ 
+   //STOPAFTER(cov->isoown==EARTH_COORD || true, PMI(cov));
+   //   assert(cov->isoown ==EARTH_ISOTROPIC || __stopafter__ ==1);
+ 
+   if (cov->key == NULL) {
+     if ((err = covCpy(&(cov->key), subphi)) != NOERROR) return err;
+     addModel(&(cov->key), GENNSST_INTERN);
+   }
+
+  if ((cov->q[0] =err = CHECK(cov->key, cov->tsdim, cov->tsdim, PosDefType, 
+			      KERNEL, SYMMETRIC,  // viel zu schwach, geht aber 
+			      //                     z.Zt. nicht anders
+			      // hinsichtlich der Koordinatentransformation
+			      // -- wird das interne modell abgefangen
+			      SCALAR, cov->role )) != NOERROR) {
+    return err;
+  }
+
+  //  PL = GLOBAL.general.Cprintlevel = 7;
+
+  
+  // MUSSS ZWINGEND ALS ZWEITES KOMMEN
+  if ((err = CHECK(subpsi, cov->tsdim, cov->tsdim, NegDefType,
+		   KERNEL, UCS, 
+		   cov->key->xdimown, cov->role)) != NOERROR) {
+     return err;
+  }
+ 
+
+  cov->finiterange = false;
+  setbackward(cov, cov->key);
+  assert(!cov->finiterange); //
+
+  cov->vdim[0] = cov->vdim[1] = 1;
+  
+  EXTRA_STORAGE;
+
+  COV_DELETE(cov->sub + 0);
+  if ((err = covCpy(cov->sub + 0, cov->key->sub[0])) != NOERROR) BUG;
+  cov->sub[0]->calling = cov;
+
+  // PMI(cov->calling);   assert(err == NOERROR);BUG;
+  // ERR(err); BUG;
+
+ 
+  // kein setbackward(cov, subpsi);
+  return NOERROR;
+}
+
