@@ -27,11 +27,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <stdio.h>
 #include <stdlib.h>
 #include "RF.h"
-#include "RandomFields.h"
  
 // z coordinate run the fastest in values, x the slowest   
 
-//#define debug_tools 1
 #define TOOLS_MEMORYERROR 501
 #define TOOLS_XERROR 502
 #define TOOLS_BIN_ERROR 503
@@ -39,7 +37,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 // naechste Zeile nur notwendig, weil atan2 in Windows nicht
 // ordentlich programmiert ist
-#define NEARBYINT(x)  floor((x) * NEARBY + 0.5) / NEARBY
+#define NEARBYINT(x)  (floor((x) * (NEARBY) + 0.5) / (NEARBY))
 // #define NEARBYINT(x)  x
 
 
@@ -188,9 +186,9 @@ SEXP fftVario3D(SEXP Coord,
     emp_vario[i]= n[i]=0.0;
   }
 
-  //print("sizeof double: %d ", (int) sizeof(res_type));
-  if (sizeof(res_type) == 8 )
-    binshift = bin[nbin] * bin[nbin] * (sizeof(res_type) == 8 ? 1e-12 : 1e-4);
+  //print("sizeof double: %d ", (int) sizeof(double));
+  if (sizeof(double) == 8 )
+    binshift = bin[nbin] * bin[nbin] * (sizeof(double) == 8 ? 1e-12 : 1e-4);
   // shift all bins to the right to overcome rounding errors
   for (i=0; i<=nbin; i++) {
     BinSq[i] = bin[i]>0 ? bin[i] * bin[i] + binshift : bin[i];
@@ -422,7 +420,7 @@ SEXP fftVario3D(SEXP Coord,
 	if (fabs(emp_vario[i]) < tolerance) emp_vario[i] = 0.0; 
       }
     } // vdim1
-  } // vdim2
+  } // vdim
   
     
   // print("C SCRIPT END !!!!!!!!! \n");
@@ -431,16 +429,16 @@ SEXP fftVario3D(SEXP Coord,
   if (err != NOERROR) {
     switch (err) {
     case TOOLS_MEMORYERROR :  
-      error("Memory alloc failed in empiricalvariogram.\n");
+      ERR("Memory alloc failed in empiricalvariogram.\n");
     case TOOLS_XERROR :  
-      error("The x coordinate may not be NULL.\n"); break;
+      ERR("The x coordinate may not be NULL.\n"); break;
     case TOOLS_BIN_ERROR :
-      error("Bin components not an increasing sequence.\n"); break;
+      ERR("Bin components not an increasing sequence.\n"); break;
     default : BUG;
     }
   }
 
-  if (BinSq!=NULL) free(BinSq);
+  FREE(BinSq);
   UNPROTECT(1);
   return back;
 }
