@@ -5,7 +5,7 @@
 
  all around the nugget effect -- needs special treatment 
 
- Copyright (C) 2001 -- 2014 Martin Schlather, 
+ Copyright (C) 2001 -- 2015 Martin Schlather, 
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -25,16 +25,11 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <math.h>
 #include <stdio.h>  
 #include <stdlib.h>
- 
 #include <R_ext/Lapack.h>
 #include "RF.h"
 #include "primitive.h"
-#include "randomshape.h"
+#include "shape_processes.h"
 
-#define NUGGET_TOL 0
-#define NUGGET_VDIM 1
-#define NUGGET_PROC_TOL (COMMON_GAUSS + 1)
-#define NUGGET_PROC_VDIM (COMMON_GAUSS + 2)
 
 bool equal(cov_model *cov, int i, int j, double *X, int dim)
 {
@@ -54,25 +49,26 @@ bool equal(cov_model *cov, int i, int j, double *X, int dim)
 
 /* nugget effect model */
 void nugget(double *x, cov_model *cov, double *v) {
-
   double diag = (*x <= P0(NUGGET_TOL)) ? 1.0 : 0.0;
   int i, endfor,
-    vdim   = cov->vdim2[0],
+    vdim   = cov->vdim[0],
     vdimsq = vdim * vdim;
-  assert(cov->vdim2[0] == cov->vdim2[1]);
+  assert(cov->vdim[0] == cov->vdim[1]);
+
 
   v[0] = diag;
   for (i = 1; i<vdimsq; v[i++] = diag) {
     endfor = i + vdim;
     for (; i<endfor; v[i++] = 0.0);
   }
+  //  printf("%f %f %d\n", *x, *v, PisNULL(NUGGET_TOL));
 }
 
 
 void covmatrix_nugget(cov_model *cov, double *v) {
   location_type *loc = Loc(cov);
   int 
-    vdim   = cov->vdim2[0];
+    vdim   = cov->vdim[0];
   long i,
     n = loc->totalpoints * vdim,
     nP1 = n + 1,
@@ -89,10 +85,12 @@ void covmatrix_nugget(cov_model *cov, double *v) {
 
 }
 
-char iscovmatrix_nugget(cov_model VARIABLE_IS_NOT_USED *cov) {  return true; }
+char iscovmatrix_nugget(cov_model VARIABLE_IS_NOT_USED *cov) {  
+  return true; 
+}
 
-void Inversenugget(double *x, cov_model VARIABLE_IS_NOT_USED *cov, double *v) { 
-  *v = (*x==1.0) ? 0.0 : RF_INF; //or better 0.0 => error?
+void Inversenugget(double VARIABLE_IS_NOT_USED *x, cov_model VARIABLE_IS_NOT_USED *cov, double *v) { 
+  *v = 0.0; ///(*x==1.0) ? 0.0 : RF_INF; //or better 0.0 => error?
 }
 
 
@@ -108,10 +106,10 @@ int check_nugget(cov_model *cov) {
   kdefault(cov, NUGGET_TOL, gp->tol);
   if (PisNULL(NUGGET_VDIM)) {
 
-    if (cov->vdim2[0] <= 0) cov->vdim2[0] = cov->vdim2[1] = 1;
-    kdefault(cov, NUGGET_VDIM, cov->vdim2[0]);
+    if (cov->vdim[0] <= 0) cov->vdim[0] = cov->vdim[1] = 1;
+    kdefault(cov, NUGGET_VDIM, cov->vdim[0]);
   } else {
-    cov->vdim2[0] = cov->vdim2[1] = P0INT(NUGGET_VDIM);
+    cov->vdim[0] = cov->vdim[1] = P0INT(NUGGET_VDIM);
   }
 
   cov->matrix_indep_of_x = true;
@@ -152,9 +150,10 @@ int check_nugget_proc(cov_model *cov) {
 
   // print("NUGGET PROC !!!! \n");
 
+  ASSERT_CARTESIAN;
   ROLE_ASSERT(ROLE_GAUSS);
  
-  if (key == NULL) { 
+   if (key == NULL) { 
     intern = sub;
     while (intern != NULL && isDollar(intern)) {
       intern = intern->key != NULL ? intern->key : intern->sub[0];
@@ -203,11 +202,12 @@ int check_nugget_proc(cov_model *cov) {
    
 
 
-  cov->vdim2[0] = next->vdim2[0];  
-  cov->vdim2[1] = next->vdim2[1];  
+  cov->vdim[0] = next->vdim[0];  
+  cov->vdim[1] = next->vdim[1];  
   if (cov->tsdim != cov->xdimprev || cov->tsdim != cov->xdimown)
     return ERRORDIM;
   cov->role = ROLE_GAUSS;
+  if ((err = kappaBoxCoxParam(cov, GAUSS_BOXCOX)) != NOERROR) return err;
   
   // printf("OK nugget\n");
   EXTRA_STORAGE;
@@ -217,6 +217,8 @@ int check_nugget_proc(cov_model *cov) {
 
 
 void range_nugget_proc(cov_model  VARIABLE_IS_NOT_USED *cov, range_type *range) {
+  GAUSS_COMMON_RANGE;
+
   range->min[NUGGET_PROC_TOL] = 0;
   range->max[NUGGET_PROC_TOL] = RF_INF;
   range->pmin[NUGGET_PROC_TOL] = 0;
@@ -234,8 +236,7 @@ void range_nugget_proc(cov_model  VARIABLE_IS_NOT_USED *cov, range_type *range)
 
 // uses global RANDOM !!!
 int init_nugget(cov_model *cov, gen_storage VARIABLE_IS_NOT_USED *S){
-  location_type 
-    *loc=cov->prevloc; 
+  location_type *loc=PrevLoc(cov); 
   if (cov->ownloc!=NULL) {
     LOC_DELETE(&(cov->ownloc));
     //PMI(cov);
@@ -244,7 +245,7 @@ int init_nugget(cov_model *cov, gen_storage VARIABLE_IS_NOT_USED *S){
   cov_model *next = cov->sub[0];
   nugget_storage *s;
   int d, //
-    vdim = cov->vdim2[0],
+    vdim = cov->vdim[0],
     err = NOERROR,
     origdim = loc->timespacedim,
     dim = cov->tsdim,
@@ -256,26 +257,21 @@ int init_nugget(cov_model *cov, gen_storage VARIABLE_IS_NOT_USED *S){
   ROLE_ASSERT_GAUSS;
 
   cov->method = Nugget;
-  NEW_STORAGE(Snugget, NUGGET, nugget_storage);
+  NEW_STORAGE(nugget);
   s = cov->Snugget;
   s->pos = NULL;
   s->red_field = NULL;
 
-  if (next->nr != NUGGET) {
-     err=ERRORCOVFAILED;
-     strcpy(ERRORSTRING_OK, CovList[NUGGET].nick);
-     strcpy(ERRORSTRING_WRONG, NICK(cov));
-     goto ErrorHandling;
-  }
-  
+  if (next->nr != NUGGET)
+    GERR2("'%s' was called by '%s'", NICK(cov), NICK(next));
 
   if ((s->simple = origdim == dim)) {
-    double value[MAXNUGGDIM], ivalue[MAXNUGGDIM], dummy[5 * MAXNUGGDIM], *A;
+    double value[MAXNUGGETDIM], ivalue[MAXNUGGETDIM], dummy[5 * MAXNUGGETDIM], *A;
     int 
       ndummy = 5 * origdim;
     char No = 'N';
 //	Upper = 'U';
-    //   PMI(cov);
+    //      APMI(cov->calling->calling);
     
     if (loc->caniso == NULL) {
       if (loc->grid) {
@@ -286,8 +282,8 @@ int init_nugget(cov_model *cov, gen_storage VARIABLE_IS_NOT_USED *S){
 	  }
       }
     } else {    
-     if (dim > MAXNUGGDIM) {
-       GERR2("dim=%d larger than MAXNUGGDIM=%d", dim, MAXNUGGDIM);
+      if (dim > MAXNUGGETDIM) {
+       GERR2("dim=%d larger than MAXNUGGETDIM=%d", dim, MAXNUGGETDIM);
       } 
       anisotype = Type(loc->caniso, loc->cani_nrow, loc->cani_ncol);
       A = (double*) MALLOC(dimSq * sizeof(double));
@@ -300,7 +296,7 @@ int init_nugget(cov_model *cov, gen_storage VARIABLE_IS_NOT_USED *S){
 		      value, ivalue, NULL, &origdim, NULL, &origdim,
 		      dummy, &ndummy, &err);
       if (err != 0) {
-	free(A);
+	FREE(A);
 	GERR1("dgeev failed for '%s'", NICK(cov));
       }
       for (d=0; d<origdim; d++) {
@@ -310,14 +306,14 @@ int init_nugget(cov_model *cov, gen_storage VARIABLE_IS_NOT_USED *S){
 	if (!(s->simple = fabs(value[d]) + fabs(ivalue[d]) > EIGENVALUE_EPS))
 	   break;
       }
-      free(A);
+      FREE(A);
     }
   }
 
   s->simugrid = loc->grid && isMdiag(anisotype);
   assert(!s->simugrid || loc->caniso == NULL);
 
-//   print("simple %d %d\n", s->simple, s->simugrid); //assert(false);
+//    print("simple %d %d; %d %d %f\n", s->simple, s->simugrid, PL, PL_IMPORTANT, tol); 
 
   if (!s->simple) {
     int *pos, oldpos;
@@ -329,13 +325,14 @@ int init_nugget(cov_model *cov, gen_storage VARIABLE_IS_NOT_USED *S){
       s->prod_dim[0] = 1; 
 
       for (d=0; d<dim; d++) {
-	s->reduced_dim[d] = fabs(loc->xgr[d][XSTEP]) <= tol ? 1 : loc->length[d];
+	s->reduced_dim[d] = 
+	  fabs(loc->xgr[d][XSTEP]) <= tol ? 1 : loc->xgr[d][XLENGTH];
 	s->prod_dim[d + 1] = s->prod_dim[d] * s->reduced_dim[d];
 	
       //print("d=%d %d %d %d\n",d,s->prod_dim[0],s->prod_dim[1],s->prod_dim[2]);
       }
 
-      if ((s->red_field=(res_type *) MALLOC(sizeof(res_type) * vdim *
+      if ((s->red_field=(double *) MALLOC(sizeof(double) * vdim *
 					    s->prod_dim[dim]))
 	  == NULL){
 	  err=ERRORMEMORYALLOCATION; goto ErrorHandling;
@@ -345,9 +342,9 @@ int init_nugget(cov_model *cov, gen_storage VARIABLE_IS_NOT_USED *S){
       if ((pos = (int*) MALLOC(sizeof(int) * loc->totalpoints)) == NULL) {
 	err=ERRORMEMORYALLOCATION; goto ErrorHandling;
       }
-      Transform2NoGrid(cov, false, true);
-      loc = cov->ownloc;
-      ordering(loc->x, loc->totalpoints, dim, pos);
+      TransformLoc(cov, false, true, true);
+      loc = Loc(cov); 
+      RU_ordering(loc->x, loc->totalpoints, dim, pos);
 
       //      for (i=0; i<loc->totalpoints; i++) {
       //	printf("old %f %f pos[i]=%d new=%f %f\n",
@@ -380,23 +377,23 @@ void do_nugget(cov_model *cov, gen_storage VARIABLE_IS_NOT_USED *S) {
   nugget_storage* s;
   long nx, endfor;
   int v, 
-    vdim = cov->vdim2[0];
+    vdim = cov->vdim[0];
   double 
     *res = cov->rf;
-  bool loggauss = GLOBAL.gauss.loggauss;
-
+  SAVE_GAUSS_TRAFO;
+ 
   s = (nugget_storage*) cov->Snugget;
 //  sqrtnugget = s->sqrtnugget;
 
   if (s->simple) {
     for (nx=0, endfor=loc->totalpoints * vdim; nx<endfor; nx++) {
-      res[nx] = (res_type) GAUSS_RANDOM(1.0);
+      res[nx] = (double) GAUSS_RANDOM(1.0);
     }
   } else {
     if (s->simugrid) {
-      int d, i, dim, dimM1, index[MAXNUGGDIM], *red_dim, *prod_dim;
+      int d, i, dim, dimM1, index[MAXNUGGETDIM], *red_dim, *prod_dim;
       long totpnts, idx;
-      res_type *field;
+      double *field;
       totpnts = loc->totalpoints;
       dim = cov->tsdim;
       dimM1 = dim -1;
@@ -410,18 +407,17 @@ void do_nugget(cov_model *cov, gen_storage VARIABLE_IS_NOT_USED *S) {
       //print("reddim %d %d dim=%d %d %d %d endfor=%d\n", red_dim[0], red_dim[1], dim, s->prod_dim[0], s->prod_dim[1], s->prod_dim[2], endfor);
 
       for (i=0; i<endfor; i++) {
-	field[i] = (res_type) GAUSS_RANDOM(1.0);
+	field[i] = (double) GAUSS_RANDOM(1.0);
       }
       for (d=0; d<dim; index[d++] = 0);
       for (i=0; i<totpnts; i++) {
 	for(idx=d=0; d<dim; d++) idx += (index[d] % red_dim[d]) * prod_dim[d];
-	//	print("for %d %d %f d=%d %d len=%d %d \n", i, idx, (double) field[idx], index[0], index[1], loc->length[0], loc->length[1]);
 	for (v=0; v<vdim; v++) {
 	  res[i + v] = field[idx + v];
 	}
 	d = 0; 
 	(index[d])++; 
-	while (d < dimM1 && index[d] >= loc->length[d]) { 
+	while (d < dimM1 && index[d] >= loc->xgr[d][XLENGTH]) { 
 	  assert(d<dim); // assert(d>=0);
 	  index[d] = 0; 
 	  d++;   
@@ -447,11 +443,8 @@ void do_nugget(cov_model *cov, gen_storage VARIABLE_IS_NOT_USED *S) {
     }
   }
 
-  if (loggauss) {
-    int i, 
-      vdimtot = loc->totalpoints * cov->vdim2[0];
-    for (i=0; i<vdimtot; i++) res[i] = exp(res[i]);
-  }
+  BOXCOX_INVERSE;
+
 }
 
 
